// C++ Tariff Class (kId)
// Copyright (C) 1995 Klaus Luft

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Created by: Klaus Luft, Klaus.Luft@stud.uni-karlsruhe.de
// Created on: Fri Jan  5 20:08:25 1996
// Last modified by: Klaus Luft
// Last modified on: Tue Feb 20 20:00:25 1996
// Filename: ktarif.cpp
// Purpose: C++ Tariff Class (kId)

#include <stdio.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#include "ktarif.h"

// Einheitentabellen (in Sekunden) Stand Januar 1996 
// zum Teil ist die Dauer einer Einheit abgerundet!
// ticks[zone][tarif]
const int ticks[4][5] = {
  // Mo   Na   Fr   Vm   Nm 
  { 240, 240, 150,  90,  90 },  // City
  {  60, 120,  45,  26,  30 },  // Region50
  {  30, 120,  21,  12,  13 },  // Region200
  {  25, 120,  20,  11,  12 }   // Fern
};

// Halbe Sekunden! TODO!
/*
const int ticks[4][5] = {
  { 480, 480, 300, 180, 180 }, 
  { 120, 240,  90,  52,  60 }, 
  {  60, 240,  43,  24,  27 }, 
  {  50, 240,  40,  23,  25 }  
};
*/

// Stunde -> Tarifzone 
// hour2tarif[0=wochentag, 1=wochenende/feiertag][stunde]
const int hour2tarif[2][24]={
    { 0, 0, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 4, 4, 4, 4, 4, 4, 2, 2, 2, 0, 0, 0 }, // Wochentag 
    { 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0 }  // Wochenende 
};

// holidays - format: "DD.MM"
static char *holidays[] = { "01.01", // New Year's Day
			    "05.04", // Good Friday
			    "01.05", // May Day
			    "27.05", // Whit Monday
			    "03.10", // Tag der deutschen Einheit
			    "01.11", // All Saint's Day
			    "20.11", // (?) Repentance Day 
			    "24.12", // Christmas (no holiday, but cheap rate)
			    "25.12", // Christmas Day
			    "26.12", // St. Stephen Day
			    "31.12"  // Sylvester (no holiday, but cheap rate)
};

kTarif::kTarif(unsigned int delay) {
  /* Description..: Constructor for kTarif class. Get rate duration. Start online timer.
     Parameter....: unsigned int delay - seconds to detect carrier */
  cd_time = delay;

  kTarifReset();
  kTarifStart();
}

kTarif::~kTarif() {
  /* Description..: Destructor for kTarif class.
     Parameter....: - */
  for(int i=0; i<4; i++)
    last[i]=0;  // Huh? 
}

void kTarif::kTarifReset(zones z) {
  /* Description..: Berechnet und setzt die Dauer einer Einheit fuer eine bestimmte Zone.
     Parameter....: zones zone - Tarifzone (city, region50, region200, fern) */
  struct tm *tmval;

  time_t t=time(NULL);
  tmval = localtime(&t);

  // Ermitteln des Tarifs
  int wday=tmval->tm_wday;
  int hour=tmval->tm_hour;
  int mday=tmval->tm_mday;
  int mon=tmval->tm_mon+1;

  feiertag = kTarifIsHoliday(mday, mon);

  if(wday == 0 || wday == 6 || feiertag)      // saturday, sunday or holiday
    dauer[int(z)]=ticks[int(z)][hour2tarif[1][hour]];
  else 
    dauer[int(z)]=ticks[int(z)][hour2tarif[0][hour]];
}

void kTarif::kTarifReset() {
  /* Description..: Berechnet und setzt die Dauer einer Einheit fuer alle Zonen. 
     Parameter....: - */
  struct tm *tmval;

  time_t t=time(NULL);
  tmval = localtime(&t);

  // Ermitteln des Tarifs
  int wday=tmval->tm_wday;
  int hour=tmval->tm_hour;
  int mday=tmval->tm_mday;
  int mon=tmval->tm_mon+1;

  feiertag = kTarifIsHoliday(mday, mon);

  if(wday == 0 || wday == 6 || feiertag) {     // saturday, sunday or holiday
    dauer[0]=ticks[0][hour2tarif[1][hour]];
    dauer[1]=ticks[1][hour2tarif[1][hour]];
    dauer[2]=ticks[2][hour2tarif[1][hour]];
    dauer[3]=ticks[3][hour2tarif[1][hour]];
  }
  else {
    dauer[0]=ticks[0][hour2tarif[0][hour]];
    dauer[1]=ticks[1][hour2tarif[0][hour]];
    dauer[2]=ticks[2][hour2tarif[0][hour]];
    dauer[3]=ticks[3][hour2tarif[0][hour]];
  }
}

void kTarif::kTarifStart() {
  /* Description..: Store begin of online time and reset misc variables.
     Parameter....: - */
  begin=time(NULL) - cd_time;
  cd_time=0;  
  online=0;

  for(int i=0; i<4; i++) {
    units[i] = 1;
    last[i] = 0;
    rest[i] = dauer[i];
  }
}

void kTarif::kTarifUpdate() {
  /* Description..: Update variables rest, online (ges_online), units (ges_units).
                    This function has to call every time before you call kTarifGet...-
		    functions (except kTarifGetZone()).
     Parameter....: - */
  struct tm *tmval;

  long diff=long(difftime(time(NULL), begin));
  int hour=int(diff/3600);
  diff=diff%3600;
  int min=int(diff/60);
  diff=diff%60;
  int sec=int(diff);
  
  online = (hour*3600+min*60+sec);
  for(int i=0; i<4; i++) {
    rest[i] = dauer[i] - online + last[i];

    if(rest[i]<=0) {  // new tick
      kTarifReset(zones(i));    // check, if new tarif
      last[i]=online + rest[i];
      rest[i]=kTarifGetDuration(zones(i));
      units[i]++;
    }
  }
}

float kTarif::kTarifGetMoney(zones z) {
  /* Description..: Returns the wasted money.
     Parameter....: - */
  float erg;
  erg=units[int(z)] * _KTARIF_MPU;

  return(erg);
}

long kTarif::kTarifGetOnline() {
  /* Description..: Returns the online time in seconds
     Parameter....: - */
  return(online);
}

int kTarif::kTarifIsHoliday(int day, int month) {
  /* Description..: Returns 1 if the specified date is a holiday, otherwise 0
     Parameter....: int day
                    int month */
  char buf[10];
  int i=0;
  int ok=1;
  sprintf(buf, "%02d.%02d", day, month);

  int anz=(sizeof (holidays))/sizeof(holidays[0]);

  while(i<anz && ok) {
    char buf2[10];
    if(strcmp(buf, holidays[i])==0)
       ok=0;
    i++;
  }
  return(!ok);
}

int kTarif::kTarifGetRTime(zones z) { 
  return(rest[int(z)]); 
}

int kTarif::kTarifGetDuration(zones z) { 
  return(dauer[int(z)]); 
}

int kTarif::kTarifGetUnits(zones z) { 
  return(units[int(z)]); 
}

// EOF: ktarif.cc

